PADDING = (
    1,
    1,
)

PADDING_LIST = (
    1,
    1,
)


class Chars:
    FILLED_CIRCLE = b'\xe2\x97\x8f'
    EMPTY_CIRCLE = b'\xe2\x97\x8b'


class ExitCodes:
    NO_SUCH_FILE_OR_DIRECTORY = 2
    FILE_EXISTS = 17


class HelpPanels:
    DISPLAY = 'Display informations'
    CHANGE = 'Change configurations'
